#include "USB.h"
#include "esp32-hal-adc.h"
#include "esp32-hal-dac.h"

#if ARDUINO_USB_CDC_ON_BOOT
#define HWSerial Serial0
#define USBSerial Serial
#else
#define HWSerial Serial
USBCDC USBSerial;
#endif

/*
 * 
 * it is 12bit SAR ADC ,8bit DAC of ESP32-S2
 * the adcx pin is GPIO6,GPIO7,GPIO8.
 * get battery ad value from adc3 of GPIO1
 * 
 * the dac_pin is attached to GPIO17,the range value is 0~255
 * 
 * 
*/
#define adc0 6
#define adc1 7
#define adc2 8
#define adc3 1
#define dac_pin 17

uint16_t adc_Value = 0;    // variable to store the value coming from the sensor
uint8_t dac_value = 127;

static void usbEventCallback(void* arg, esp_event_base_t event_base, int32_t event_id, void* event_data){
  if(event_base == ARDUINO_USB_EVENTS){
    arduino_usb_event_data_t * data = (arduino_usb_event_data_t*)event_data;
    switch (event_id){
      case ARDUINO_USB_STARTED_EVENT:
        HWSerial.println("USB PLUGGED");
        break;
      case ARDUINO_USB_STOPPED_EVENT:
        HWSerial.println("USB UNPLUGGED");
        break;
      case ARDUINO_USB_SUSPEND_EVENT:
        HWSerial.printf("USB SUSPENDED: remote_wakeup_en: %u\n", data->suspend.remote_wakeup_en);
        break;
      case ARDUINO_USB_RESUME_EVENT:
        HWSerial.println("USB RESUMED");
        break;
      
      default:
        break;
    }
  } else if(event_base == ARDUINO_USB_CDC_EVENTS){
    arduino_usb_cdc_event_data_t * data = (arduino_usb_cdc_event_data_t*)event_data;
    switch (event_id){
      case ARDUINO_USB_CDC_CONNECTED_EVENT:
        HWSerial.println("CDC CONNECTED");
        break;
      case ARDUINO_USB_CDC_DISCONNECTED_EVENT:
        HWSerial.println("CDC DISCONNECTED");
        break;
      case ARDUINO_USB_CDC_LINE_STATE_EVENT:
        HWSerial.printf("CDC LINE STATE: dtr: %u, rts: %u\n", data->line_state.dtr, data->line_state.rts);
        break;
      case ARDUINO_USB_CDC_LINE_CODING_EVENT:
        HWSerial.printf("CDC LINE CODING: bit_rate: %u, data_bits: %u, stop_bits: %u, parity: %u\n", data->line_coding.bit_rate, data->line_coding.data_bits, data->line_coding.stop_bits, data->line_coding.parity);
        break;
      case ARDUINO_USB_CDC_RX_EVENT:
        HWSerial.printf("CDC RX [%u]:", data->rx.len);
        {
            uint8_t buf[data->rx.len];
            size_t len = USBSerial.read(buf, data->rx.len);
            HWSerial.write(buf, len);
        }
        HWSerial.println();
        break;
      
      default:
        break;
    }
  }
}

void setup() {
  // put your setup code here, to run once:
  HWSerial.begin(115200);
  HWSerial.setDebugOutput(true);
  
  USB.onEvent(usbEventCallback);
  USBSerial.onEvent(usbEventCallback);
  
  USBSerial.begin();
  USB.begin();

  delay(1000);
  delay(1000);
  delay(1000);
  delay(1000);
//  USBSerial.printf("Setup done");
}

void loop() {
  // put your main code here, to run repeatedly:
  
/*
 * you can wire adcx pin to dac pin to test AD/DA.
 */
  dac_value++;
  dacWrite(dac_pin,dac_value);
  adc_Value = analogRead(adc0);
  USBSerial.printf("ADC value of GPIO6 is %d\n",adc_Value);
  USBSerial.printf("convert to voltage is %fV\n",adc_Value/8191.0*3.3);
  adc_Value = analogRead(adc1);
  USBSerial.printf("ADC value of GPIO7 is %d\n",adc_Value);
  USBSerial.printf("convert to voltage is %fV\n",adc_Value/8191.0*3.3);
  adc_Value = analogRead(adc2);
  USBSerial.printf("ADC value of GPIO8 is %d\n",adc_Value);
  USBSerial.printf("convert to voltage is %fV\n",adc_Value/8191.0*3.3);
  
/*  
 *  Get battery volatage,please solder the 0R resistor on R37 position 
 *  and remove the R36(0R).
 *  
 *  please subtract bias(0.34) when get AD value of GPIO1, because the
 *  5% precision resistor brings about to bias, we will fix it in the 
 *  next version
 *    
 */
  adc_Value = analogRead(adc3);
  USBSerial.printf("ADC value of GPIO1 is %d\n",adc_Value);
  USBSerial.printf("convert to battery voltage is %fV\n",(adc_Value/8191.0*3.3-0.34)*3);
  
  delay(5000);
}
