import bitbangio
import busio
import digitalio
from board import *
from adafruit_bus_device.spi_device import SPIDevice
import displayio
import time

#color is BGR
RED = 0x00F8
GREEN = 0xE007
BLUE = 0x1F00
WHITE = 0xFFFF
BLACK = 0x0000

class LCD_0inch96():
    def __init__(self):

        self.width = 160
        self.height = 80
        
        self.cs = digitalio.DigitalInOut(IO9)
        self.cs.direction = digitalio.Direction.OUTPUT
        
        self.rst = digitalio.DigitalInOut(IO21)
        self.rst.direction = digitalio.Direction.OUTPUT
        
        self.bl = digitalio.DigitalInOut(IO45)
        self.bl.direction = digitalio.Direction.OUTPUT
        
        self.dc = digitalio.DigitalInOut(IO18)
        self.dc.direction = digitalio.Direction.OUTPUT
        
        self.spi = busio.SPI(IO10,IO11,IO12)
        
        self.cs.value=True
        self.dc.value=True
        self.bl.value=True
    
        self.Init()
        self.SetWindows(0, 0, self.width-1, self.height-1)
        
    def reset(self):
        self.rst.value=True
        time.sleep(0.2) 
        self.rst.value=False
        time.sleep(0.2)
        self.rst.value=True
        time.sleep(0.2) 
        
    def write_cmd(self, cmd):
        self.dc.value=False
        self.cs.value=False
        if self.spi.try_lock()==True:
            self.spi.write(bytearray([cmd]))
        self.spi.unlock()

    def write_data(self, buf):
        self.dc.value=True
        self.cs.value=False
        if self.spi.try_lock()==True:
            self.spi.write(bytearray([buf]))
        self.spi.unlock()    
        self.cs.value=True

    def Init(self):
        self.reset() 

        self.write_cmd(0x11)
        time.sleep(0.12)
        self.write_cmd(0x21) 
        self.write_cmd(0x21) 

        self.write_cmd(0xB1) 
        self.write_data(0x05)
        self.write_data(0x3A)
        self.write_data(0x3A)

        self.write_cmd(0xB2)
        self.write_data(0x05)
        self.write_data(0x3A)
        self.write_data(0x3A)

        self.write_cmd(0xB3) 
        self.write_data(0x05)  
        self.write_data(0x3A)
        self.write_data(0x3A)
        self.write_data(0x05)
        self.write_data(0x3A)
        self.write_data(0x3A)

        self.write_cmd(0xB4)
        self.write_data(0x03)

        self.write_cmd(0xC0)
        self.write_data(0x62)
        self.write_data(0x02)
        self.write_data(0x04)

        self.write_cmd(0xC1)
        self.write_data(0xC0)

        self.write_cmd(0xC2)
        self.write_data(0x0D)
        self.write_data(0x00)

        self.write_cmd(0xC3)
        self.write_data(0x8D)
        self.write_data(0x6A)   

        self.write_cmd(0xC4)
        self.write_data(0x8D) 
        self.write_data(0xEE) 

        self.write_cmd(0xC5)
        self.write_data(0x0E)

        self.write_cmd(0xE0)
        self.write_data(0x10)
        self.write_data(0x0E)
        self.write_data(0x02)
        self.write_data(0x03)
        self.write_data(0x0E)
        self.write_data(0x07)
        self.write_data(0x02)
        self.write_data(0x07)
        self.write_data(0x0A)
        self.write_data(0x12)
        self.write_data(0x27)
        self.write_data(0x37)
        self.write_data(0x00)
        self.write_data(0x0D)
        self.write_data(0x0E)
        self.write_data(0x10)

        self.write_cmd(0xE1)
        self.write_data(0x10)
        self.write_data(0x0E)
        self.write_data(0x03)
        self.write_data(0x03)
        self.write_data(0x0F)
        self.write_data(0x06)
        self.write_data(0x02)
        self.write_data(0x08)
        self.write_data(0x0A)
        self.write_data(0x13)
        self.write_data(0x26)
        self.write_data(0x36)
        self.write_data(0x00)
        self.write_data(0x0D)
        self.write_data(0x0E)
        self.write_data(0x10)

        self.write_cmd(0x3A) 
        self.write_data(0x05)

        self.write_cmd(0x36)
        self.write_data(0xA8)

        self.write_cmd(0x29) 
        
    def SetWindows(self, Xstart, Ystart, Xend, Yend):#example max:0,0,159,79
        Xstart=Xstart+1
        Xend=Xend+1
        Ystart=Ystart+26
        Yend=Yend+26
        self.write_cmd(0x2A)
        self.write_data(0x00)              
        self.write_data(Xstart)      
        self.write_data(0x00)              
        self.write_data(Xend) 

        self.write_cmd(0x2B)
        self.write_data(0x00)
        self.write_data(Ystart)
        self.write_data(0x00)
        self.write_data(Yend)

        self.write_cmd(0x2C) 

    def display(self):
        self.SetWindows(0,0,self.width-1,self.height-1)
        self.dc.value=True
        self.cs.value=False
        if self.spi.try_lock()==True:        
            self.spi.write(bytearray(0xF0 for _ in range(8000)))
        self.spi.unlock()
        self.cs.value=True

if __name__=='__main__':

    lcd = LCD_0inch96()
    lcd.display()


