/*
 * NMEA0183 library is from https://github.com/ttlappalainen/NMEA0183
 * Thanks to ttlappalainen 
 */
#include "NMEA0183.h"
#include "NMEA0183Msg.h"
#include "NMEA0183Messages.h"
#include "USB.h"

#if ARDUINO_USB_CDC_ON_BOOT
#define HWSerial Serial0
#define USBSerial Serial
#else
#define HWSerial Serial
USBCDC USBSerial;
#endif

tNMEA0183Msg NMEA0183Msg;
tNMEA0183 NMEA0183;

#define RXD2 3
#define TXD2 2

static void usbEventCallback(void* arg, esp_event_base_t event_base, int32_t event_id, void* event_data){
  if(event_base == ARDUINO_USB_EVENTS){
    arduino_usb_event_data_t * data = (arduino_usb_event_data_t*)event_data;
    switch (event_id){
      case ARDUINO_USB_STARTED_EVENT:
        HWSerial.println("USB PLUGGED");
        break;
      case ARDUINO_USB_STOPPED_EVENT:
        HWSerial.println("USB UNPLUGGED");
        break;
      case ARDUINO_USB_SUSPEND_EVENT:
        HWSerial.printf("USB SUSPENDED: remote_wakeup_en: %u\n", data->suspend.remote_wakeup_en);
        break;
      case ARDUINO_USB_RESUME_EVENT:
        HWSerial.println("USB RESUMED");
        break;
      
      default:
        break;
    }
  } else if(event_base == ARDUINO_USB_CDC_EVENTS){
    arduino_usb_cdc_event_data_t * data = (arduino_usb_cdc_event_data_t*)event_data;
    switch (event_id){
      case ARDUINO_USB_CDC_CONNECTED_EVENT:
        HWSerial.println("CDC CONNECTED");
        break;
      case ARDUINO_USB_CDC_DISCONNECTED_EVENT:
        HWSerial.println("CDC DISCONNECTED");
        break;
      case ARDUINO_USB_CDC_LINE_STATE_EVENT:
        HWSerial.printf("CDC LINE STATE: dtr: %u, rts: %u\n", data->line_state.dtr, data->line_state.rts);
        break;
      case ARDUINO_USB_CDC_LINE_CODING_EVENT:
        HWSerial.printf("CDC LINE CODING: bit_rate: %u, data_bits: %u, stop_bits: %u, parity: %u\n", data->line_coding.bit_rate, data->line_coding.data_bits, data->line_coding.stop_bits, data->line_coding.parity);
        break;
      case ARDUINO_USB_CDC_RX_EVENT:
        HWSerial.printf("CDC RX [%u]:", data->rx.len);
        {
            uint8_t buf[data->rx.len];
            size_t len = USBSerial.read(buf, data->rx.len);
            HWSerial.write(buf, len);
        }
        HWSerial.println();
        break;
      
      default:
        break;
    }
  }
}

void setup() {
  // put your setup code here, to run once:
  Serial1.begin(9600,SERIAL_8N1,RXD2,TXD2);
  NMEA0183.Begin(&Serial1,1, 9600);

  HWSerial.begin(115200);
  HWSerial.setDebugOutput(true);
  
  USB.onEvent(usbEventCallback);
  USBSerial.onEvent(usbEventCallback);
  
  USBSerial.begin();
  USB.begin();

  delay(1000);
  delay(1000);
  delay(1000);
  delay(1000);
}

void loop() {
  // put your main code here, to run repeatedly:
  tNMEA0183Msg NMEA0183Msg;
  while (NMEA0183.GetMessage(NMEA0183Msg)) {
  USBSerial.printf(NMEA0183Msg.Sender());
  USBSerial.printf(NMEA0183Msg.MessageCode());
  for (int i=0; i < NMEA0183Msg.FieldCount(); i++) {
    USBSerial.printf(NMEA0183Msg.Field(i));
  }
  USBSerial.printf("\n");

   /*
    * To Be Done
    * WGS84 coordinate convert to GCJ02 coordinate
    */
    
  }
}
